@@ -377,9 +377,10 @@ xcircuit::tag promptchecksavelib {xcircuit::promptsavelib %r}
 xcircuit::tag promptsavepage {xcircuit::pageupdate ; wm deiconify .output ; \
 	raise .output}
 xcircuit::tag loadfont {xcircuit::newfontbutton %r}
-xcircuit::tag color {set colorval %1; set iname img_co; \
-	if {"%1" != "inherit"} {append iname l%1} ; \
-	.xcircuit.mainframe.toolbar.bco configure -image $iname}
+xcircuit::tag color { if {"%1" == "set"} { \
+	set colorval %2; set iname img_co; \
+	if {"%2" != "inherit"} {append iname l%2} ; \
+	.xcircuit.mainframe.toolbar.bco configure -image $iname} }
 xcircuit::tag fill { switch -- "%1" { opaque { set opaque true } \
 	   transparent { set opaque false } 0 - unfilled {set fillamount 0; \
 	      .xcircuit.mainframe.toolbar.bfi configure -image img_fi} \
@@ -387,6 +388,7 @@ xcircuit::tag fill { switch -- "%1" { opaque { set opaque true } \
 	      .xcircuit.mainframe.toolbar.bfi configure -image img_stip100}
 	   default {set fillamount %1; \
 	      .xcircuit.mainframe.toolbar.bfi configure -image img_stip%1} } }
+xcircuit::tag parameter { if {"%1" == "set"} {xcircuit::prompteditparams substring} }
 
 #------------------------------
 # Create the file-list window
@@ -646,21 +648,39 @@ grid rowconfigure .help.listwin 0 -weight 1 -minsize 100
 #-----------------------------------------------------------------
 
 proc xcircuit::reset_select_listbox {} {
-   if {[catch {wm withdraw .select}]} {
+   if {[catch {wm state .select}]} {
       toplevel .select -bg beige
-      wm withdraw .select
 
-      listbox .select.sellist -bg white
+      label .select.title -text "Parameters" -bg beige
+      label .select.keytitle -text "Key" -bg beige
+      label .select.valtitle -text "Value" -bg beige
+
+      listbox .select.keylist -bg white
       listbox .select.vallist -bg white
-      pack .select.sellist -side left -expand true -fill both
-      pack .select.vallist -side left -expand true -fill both
-      bind .select.sellist <ButtonRelease-1> {set p_key [.select.sellist get \
-		[.select.sellist nearest %y]]}
-      bind .select.sellist <ButtonRelease-3> {set p_key {}}
+
+      button .select.dismiss -text "Dismiss" -bg beige \
+		-command {wm withdraw .select}
+
+      grid .select.title -row 0 -column 0 -columnspan 2 -sticky news
+      grid .select.keytitle -row 1 -column 0 -sticky news
+      grid .select.keylist -row 2 -column 0 -sticky news
+      grid .select.valtitle -row 1 -column 1 -sticky news
+      grid .select.vallist -row 2 -column 1 -sticky news
+      grid .select.dismiss -row 3 -column 0 -columnspan 2 -sticky ns
+
+      grid rowconfigure .select 2 -weight 1
+      grid columnconfigure .select 0 -weight 1
+      grid columnconfigure .select 1 -weight 2
+
+      bind .select.keylist <ButtonRelease-1> {set p_key [.select.keylist get \
+		[.select.keylist nearest %y]]}
+      bind .select.vallist <ButtonRelease-1> {set p_key [.select.keylist get \
+		[.select.keylist nearest %y]]}
+      bind .select.keylist <ButtonRelease-3> {set p_key {}}
       bind .select.vallist <ButtonRelease-3> {set p_key {}}
    } else {
-      while {[.select.sellist size] > 0} {select.sellist delete 0}
-      while {[.select.vallist size] > 0} {select.vallist delete 0}
+      while {[.select.keylist size] > 0} {.select.keylist delete 0}
+      while {[.select.vallist size] > 0} {.select.vallist delete 0}
    }
 }
 
@@ -735,6 +755,20 @@ proc xcircuit::printstring {stringlist} {
    return $p
 }
 
+proc xcircuit::printjust {justif} {
+   switch [expr {$justif & 3}] {
+      0 {set p "left"}
+      1 {set p "center"}
+      3 {set p "right"}
+   }
+   switch [expr {$justif & 12}] {
+      0 {append p " bottom"}
+      4 {append p " middle"}
+      12 {append p " top"}
+   }
+   return $p
+}
+
 proc xcircuit::promptmakeparam {} {
    if {[xcircuit::select] > 0} {
       .dialog.bbar.okay configure -command \
@@ -747,6 +781,17 @@ proc xcircuit::promptmakeparam {} {
    }
 }
 
+proc xcircuit::promptchangeparam {key current} {
+  .dialog.bbar.okay configure -command \
+	[subst {xcircuit::parameter set $key \[.dialog.textent.txt get\] -forward;\
+	  wm withdraw .dialog}]
+  .dialog.textent.title.field configure -text "Enter new value for parameter $key:"
+  .dialog.textent.txt delete 0 end
+  .dialog.textent.txt insert 0 $current
+  wm deiconify .dialog
+  focus .dialog.textent.txt
+}
+
 proc xcircuit::promptselectparam {} {
    global p_key
    set dlist [xcircuit::parameter get substring]
@@ -757,16 +802,21 @@ proc xcircuit::promptselectparam {} {
       if {[llength $dlist] == 1} {
 	 set p_key [lindex [lindex $dlist 0] 0]
       } else {
+	 if {[catch {set oname [xcircuit::object name]}]} {
+	    .select.title configure -text "Parameters"
+	 } else {
+	    .select.title configure -text "Parameters of $oname"
+	 }
          foreach i $dlist {
 	    set p_name [lindex $i 0]
 	    set p_val [lindex $i 1]
-            .select.sellist insert end $p_name
+            .select.keylist insert end $p_name
 	    .select.vallist insert end [xcircuit::printstring $p_val]
          }
 	 wm deiconify .select
 	 raise .select
          tkwait variable p_key
-	 .select.sellist delete 0 end
+	 .select.keylist delete 0 end
 	 .select.vallist delete 0 end
 	 wm withdraw .select
       }
@@ -774,13 +824,40 @@ proc xcircuit::promptselectparam {} {
    return $p_key
 }
 
-proc xcircuit::prompteditparams {} {
+proc xcircuit::prompteditparams { {mode "none"} } {
    xcircuit::reset_select_listbox
-   set dlist [xcircuit::parameter get -forward]
+   if {$mode == "none"} {
+      set dlist [xcircuit::parameter get -forward]
+   } else {
+      set dlist [xcircuit::parameter get $mode -forward]
+   }
+
+   bind .select.vallist <ButtonRelease-1> {
+		set kidx [.select.keylist nearest %y]; \
+		xcircuit::promptchangeparam \
+		[.select.keylist get $kidx] [.select.vallist get $kidx] }
+
    if {[llength $dlist] > 0} {
+      if {[catch {set oname [xcircuit::object name]}]} {
+	 .select.title configure -text "Parameters"
+      } else {
+	 .select.title configure -text "Parameters of $oname"
+      }
       foreach i $dlist {
-         .select.sellist insert end [lindex $i 0]
-	 .select.vallist insert end [lindex $i 1]
+	 set p_name [lindex $i 0]
+	 set p_val [lindex $i 1]
+         .select.keylist insert end $p_name
+	 switch -- [xcircuit::parameter type $p_name -forward] {
+	    "substring" {
+	       .select.vallist insert end [xcircuit::printstring $p_val]
+	    }
+	    "justification" {
+	       .select.vallist insert end [xcircuit::printjust $p_val]
+	    }
+	    default {
+	       .select.vallist insert end $p_val
+            }
+	 }
       }
       wm deiconify .select
       raise .select
@@ -984,14 +1061,21 @@ proc xcircuit::prompttextsize {} {
    focus .dialog.textent.txt
 }
 
+# newcolorbutton is called internally to xcircuit---don't mess with it!
+
 proc xcircuit::newcolorbutton {r g b idx} {
    global XCIRCUIT_LIB_DIR
-   ::set colorrgb [format "#%04X%04X%04X" $r $g $b]
+   set colorrgb [format "#%04X%04X%04X" $r $g $b]
    image create bitmap img_col$idx -foreground $colorrgb -file \
 		${XCIRCUIT_LIB_DIR}/pixmaps/solid.xbm
    .xcircuit.menubar.optionsbutton.optionsmenu.elementsmenu.colormenu \
 	add radio -image img_col$idx -activebackground $colorrgb \
-	-variable colorval -value $idx -command "xcircuit::color $idx"
+	-variable colorval -value $idx -command "xcircuit::color set $idx"
+}
+
+proc xcircuit::picknewcolor {} {
+   set colorrgb [tkColorDialog]
+   xcircuit::color add $colorrgb
 }
 
 proc xcircuit::newencodingbutton {encodingname} {
@@ -1523,34 +1607,59 @@ $m3 add radio -label "Transparent" -variable opaque -value false \
 	-command {xcircuit::fill transparent}
 
 set m3 [menu $m2.colormenu -tearoff 0]
-$m3 add command -label "Add New Color"
+$m3 add command -label "Add New Color" -command {xcircuit::picknewcolor}
 $m3 add separator
 $m3 add radio -label "Inherit Color" -variable colorval -value inherit \
-  -command {color $colorval}
+  -command {color set $colorval}
 
 set m3 [menu $m2.parammenu -tearoff 0]
+$m3 add command -label "Edit Parameters" -command \
+	{xcircuit::prompteditparams substring}
+$m3 add separator
 $m3 add check -label "Substring" -variable substringparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make substring}
+	-offvalue false -command {if {$substringparam} \
+		{xcircuit::parameter make substring} \
+		{xcircuit::parameter forget substring}}
 $m3 add check -label "Position" -variable positionparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make position}
+	-offvalue false -command {if {$positionparam} \
+		{xcircuit::parameter make position} \
+		{xcircuit::parameter forget position}}
 $m3 add check -label "Style" -variable styleparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make style}
+	-offvalue false -command {if {$styleparam} \
+		{xcircuit::parameter make style} \
+		{xcircuit::parameter forget style}}
 $m3 add check -label "Justification" -variable justparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make justification}
+	-offvalue false -command {if {$justparam} \
+		{xcircuit::parameter make justification} \
+		{xcircuit::parameter forget justification}}
 $m3 add check -label "Start Angle" -variable startparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make "start angle"}
+	-offvalue false -command {if {$startparam} \
+		{xcircuit::parameter make "start angle"} \
+		{xcircuit::parameter forget "start angle"}}
 $m3 add check -label "End Angle" -variable endparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make "end angle"}
+	-offvalue false -command {if {$endparam} \
+		{xcircuit::parameter make "end angle"} \
+		{xcircuit::parameter forget "end angle"}}
 $m3 add check -label "Radius" -variable radiusparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make radius}
+	-offvalue false -command {if {$radiusparam} \
+		{xcircuit::parameter make radius} \
+		{xcircuit::parameter forget radius}}
 $m3 add check -label "Minor Axis" -variable minorparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make "minor axis"}
+	-offvalue false -command {if {$minorparam} \
+		{xcircuit::parameter make "minor axis"} \
+		{xcircuit::parameter forget "minor axis"}}
 $m3 add check -label "Rotation" -variable rotationparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make rotation}
+	-offvalue false -command {if {$rotationparam} \
+		{xcircuit::parameter make rotation} \
+		{xcircuit::parameter forget rotation}}
 $m3 add check -label "Scale" -variable scaleparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make scale}
+	-offvalue false -command {if {$scaleparam} \
+		{xcircuit::parameter make scale} \
+		{xcircuit::parameter forget scale}}
 $m3 add check -label "Linewidth" -variable linewidthparam -onvalue true \
-	-offvalue false -command {xcircuit::parameter make linewidth}
+	-offvalue false -command {if {$linewidthparam} \
+		{xcircuit::parameter make linewidth} \
+		{xcircuit::parameter forget linewidth}}
 
 set m3 [menu $m2.polyeditmenu -tearoff 0]
 $m3 add radio -label "Manhattan Box Edit" -variable polyedittype \
@@ -1679,6 +1788,8 @@ bind .xcircuit.mainframe.mainarea.drawing <KeyPress> {standardaction %k key %s}
 
 bind .xcircuit.mainframe.mainarea.drawing <Key-Next> {catch {page [expr {[page] + 1}]}}
 bind .xcircuit.mainframe.mainarea.drawing <Key-Prior> {catch {page [expr {[page] - 1}]}}
+bind .xcircuit.mainframe.mainarea.drawing <Control-Key-p> \
+	{xcircuit::prompteditparams substring}
 
 # Wait for the drawing area to become visible, and set the focus on it.
 
