@@ -397,8 +397,12 @@ void autostop(xcWidget w, caddr_t clientdata, caddr_t nulldata)
 #endif
 
 /*--------------------------------------------------------------*/
-/* Auto-scale							*/
+/* Auto-scale the drawing to fit the declared page size.	*/
 /* (for now, with fixed 1" margins)				*/
+/*								*/
+/* If the page is declared encapsulated, then do nothing.	*/
+/* If a frame box is on the page, then scale to fit the frame	*/
+/* to the declared page size, not the whole object.		*/
 /*--------------------------------------------------------------*/
 
 void autoscale(int page)
@@ -408,6 +412,7 @@ void autoscale(int page)
 	== CM) ? CMSCALE : INCHSCALE;
    float margin = 144.0;
    int width, height;
+   polyptr framebox;
 
    if (!(xobjs.pagelist[page]->pmode & 2)) return;
    else if (topobject->bbox.width == 0 || topobject->bbox.height == 0) {
@@ -420,13 +425,30 @@ void autoscale(int page)
    newyscale = (xobjs.pagelist[page]->pagesize.y - margin)
 	/ scalefudge;
 
+   if ((framebox = checkforbbox(topobject)) != NULL) {
+      int i, minx, miny, maxx, maxy;
+      
+      minx = maxx = framebox->points[0].x;
+      miny = maxy = framebox->points[0].y;
+      for (i = 1; i < framebox->number; i++) {
+	 if (framebox->points[i].x < minx) minx = framebox->points[i].x;
+	 else if (framebox->points[i].x > maxx) maxx = framebox->points[i].x;
+	 if (framebox->points[i].y < miny) miny = framebox->points[i].y;
+	 else if (framebox->points[i].y > maxy) maxy = framebox->points[i].y;
+      }
+      width = (maxx - minx);
+      height = (maxy - miny);
+   }
+   else {
+
 #ifdef SCHEMA
-   width = toplevelwidth(areastruct.topinstance);
-   height = toplevelheight(areastruct.topinstance);
+      width = toplevelwidth(areastruct.topinstance, NULL);
+      height = toplevelheight(areastruct.topinstance, NULL);
 #else
-   width = topobject->bbox.width;
-   height = topobject->bbox.height;
+      width = topobject->bbox.width;
+      height = topobject->bbox.height;
 #endif
+   }
 
    if (xobjs.pagelist[page]->orient == 0) {	/* Portrait */
       newxscale /= width;
@@ -436,7 +458,6 @@ void autoscale(int page)
       newxscale /= height;
       newyscale /= width;
    }
-
    xobjs.pagelist[page]->outscale = min(newxscale, newyscale);
 }
 
