@@ -259,14 +259,15 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
 /* If the indicated string part is text and the part following the	*/
 /* indicated string part is also text, merge the two.  The indicated	*/
 /* string part is returned, and the following part is freed.		*/
+/*									*/
+/* (Fixes thanks to Petter Larsson 11/17/03)				*/
 /*----------------------------------------------------------------------*/
 
 stringpart *mergestring(stringpart *firststr)
 {
-   stringpart *nextstr;
+   stringpart *nextstr = NULL;
 
-   if (!firststr) return NULL;
-   nextstr = firststr->nextpart;
+   if (firststr) nextstr = firststr->nextpart;
    if (nextstr != NULL) {
       if (firststr->type == TEXT_STRING && nextstr->type == TEXT_STRING) {
          firststr->nextpart = nextstr->nextpart;
@@ -275,10 +276,9 @@ stringpart *mergestring(stringpart *firststr)
          strcat(firststr->data.string, nextstr->data.string);
          free(nextstr->data.string);
          free(nextstr);
-         return firststr;
       }
    }
-   return NULL;
+   return firststr;
 }
 
 /*----------------------------------------------------------------------*/
@@ -289,6 +289,7 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 {
    char *key;
    stringpart *nextptr = NULL, *tmpptr;
+   static stringpart *promote = NULL;
    oparamptr ops;
 
    if (strstart->type != PARAM_START) return NULL;
@@ -312,8 +313,24 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
       }
    }
 
-   if (ops->type != XC_STRING)
-      goto linkerror;
+   if (ops->type != XC_STRING) {
+
+      /* Generate static string for promoting numerical parameters */
+      if (promote == NULL) {
+	 tmpptr = makesegment(&promote, NULL);
+	 tmpptr->type = TEXT_STRING;
+	 tmpptr->data.string = (char *)malloc(13);
+	 tmpptr = makesegment(&promote, NULL);
+	 tmpptr->type = PARAM_END;
+      }
+
+      /* Promote numerical type to string */
+      if (ops->type == XC_INT)
+	 sprintf(promote->data.string, "%12d", ops->parameter.ivalue);
+      else
+	 sprintf(promote->data.string, "%f", ops->parameter.fvalue);
+      nextptr = promote;
+   }
    else
       nextptr = ops->parameter.string;
 
@@ -329,10 +346,6 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
       return nextptr;
    }
    return NULL;
-
-linkerror:
-   Fprintf(stderr, "linkstring() error: parameter %s is not a substring!\n", key);
-   return strstart->nextpart;
 }
 
 /*----------------------------------------------------------------------*/
